package com.javacodegeeks.examples.concurrent;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.RecursiveTask;

/**
 * @author Chandan Singh
 */
public class SearchDirectory extends RecursiveTask<List<String>>
{

    private static final long serialVersionUID = 7570286838840187368L;
    
    private String directoryName = "";
    
    private String searchString = "";
    
    public SearchDirectory(String directoryName, String searchString)
    {
    	this.directoryName = directoryName;
    	this.searchString = searchString;
    }

    @Override
    protected List<String> compute()
    {
	List<String> matchingFilesList = new ArrayList<>();
	List<SearchDirectory> taskList = new ArrayList<>();
	File directory = new File(directoryName);
				
	if(directoryName == null || "".equals(directoryName) || !directory.exists())
	      throw new IllegalArgumentException("Directory Name is NOT Valid");
				
	File[] fileArray = directory.listFiles();
	for(File file : fileArray)
	{
	     if(file.isDirectory())
	     {
		SearchDirectory searchDirectory = new SearchDirectory(directoryName,searchString);
		searchDirectory.fork();
		taskList.add(searchDirectory);
	     }
	     else
            {
		if(checkName(file.getName()))
		   matchingFilesList.add(file.getPath());
            }
	}
	for(SearchDirectory sd : taskList)
	{
		List<String> intermediateResultList = sd.join();
		matchingFilesList.addAll(intermediateResultList);
	}
						
	return matchingFilesList;
    }
		
    private boolean checkName(String filename)
    {
	return filename.contains(searchString);
    }
}
